// Code generated by command: go run asm.go -out issue89.s -stubs stub.go. DO NOT EDIT.

#include "textflag.h"

// func Issue89() uint64
TEXT ·Issue89(SB), NOSPLIT, $0-8
	MOVQ $0x0000002a, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	XORQ CX, CX
	ADDQ CX, AX
	MOVQ AX, ret+0(FP)
	RET
