// Code generated by command: go run asm.go -out returns.s -stubs stub.go. DO NOT EDIT.

package returns

// Interval returns the (start, end) of an interval with the given start and size.
// Demonstrates multiple unnamed return values.
func Interval(start uint64, size uint64) (uint64, uint64)

// Butterfly performs a 2-dimensional butterfly operation: computes (x0+x1, x0-x1).
// Demonstrates multiple named return values.
func Butterfly(x0 float64, x1 float64) (y0 float64, y1 float64)

// Septuple returns an array of seven of the given byte.
// Demonstrates returning array values.
func Septuple(byte) [7]byte

// CriticalLine returns the complex value 0.5 + it on Riemann's critical line.
// Demonstrates returning complex values.
func CriticalLine(t float64) complex128

// NewStruct initializes a Struct value.
// Demonstrates returning struct values.
func NewStruct(w uint16, p [2]float64, q uint64) Struct
