// Code generated by command: go run asm.go -out pragma.s -stubs stub.go. DO NOT EDIT.

package pragma

// Add adds the values at x and y and writes the result to z.
//go:noescape
func Add(z *uint64, x *uint64, y *uint64)
