// Code generated by command: go run asm.go -pkg custom -out issue69.s -stubs stub.go. DO NOT EDIT.

#include "textflag.h"

// func Issue68() uint64
TEXT ·Issue68(SB), NOSPLIT, $0-8
	MOVQ $0x00000044, AX
	MOVQ AX, ret+0(FP)
	RET
