// Package ext is used as a target package in the external types example.
package ext

// Struct is the target type in the external types example.
type Struct struct {
	A [3]uint16
	B byte
	C string
}
