// Code generated by command: go run asm.go -out geohash.s -stubs stub.go. DO NOT EDIT.

package geohash

// EncodeInt computes the 64-bit integer geohash of (lat, lng).
func EncodeInt(lat float64, lng float64) uint64
