// Code generated by command: go run asm.go -out issue65.s -stubs stub.go. DO NOT EDIT.

#include "textflag.h"

// func Issue65()
// Requires: AVX2
TEXT ·Issue65(SB), NOSPLIT, $0
	VINSERTI128 $0x01, X0, Y1, Y2
	RET
