// Code generated by command: go run asm.go -out complex.s -stubs stub.go. DO NOT EDIT.

package complex

// Real returns the real part of z.
func Real(z complex128) float64

// Imag returns the imaginary part of z.
func Imag(z complex128) float64

// Norm returns the complex norm of z.
func Norm(z complex128) float64
