// Code generated by command: go run asm.go -out labels.s -stubs stub.go. DO NOT EDIT.

#include "textflag.h"

// func Labels() uint64
TEXT ·Labels(SB), NOSPLIT, $0-8
	XORQ AX, AX
	INCQ AX
	INCQ AX
	INCQ AX
	INCQ AX
	INCQ AX
	INCQ AX
	INCQ AX
	MOVQ AX, ret+0(FP)
	RET
