// Code generated by command: go run asm.go -out issue50.s -stubs stub.go. DO NOT EDIT.

package issue50

// Issue50 reported that MOVD/MOVQ was missing the r32, xmm form.
// This function deliberately exercises this instruction form.
func Issue50(x uint32) uint32
