// Code generated by command: go run asm.go -out data.s -stubs stub.go. DO NOT EDIT.

#include "textflag.h"

DATA bytes<>+0(SB)/8, $0x0011223344556677
DATA bytes<>+8(SB)/8, $"strconst"
DATA bytes<>+16(SB)/4, $(3.1415927)
DATA bytes<>+24(SB)/8, $(3.141592653589793)
DATA bytes<>+32(SB)/4, $0x00112233
DATA bytes<>+36(SB)/2, $0x4455
DATA bytes<>+38(SB)/1, $0x66
DATA bytes<>+39(SB)/1, $0x77
GLOBL bytes<>(SB), RODATA|NOPTR, $40

// func DataAt(i int) byte
TEXT ·DataAt(SB), NOSPLIT, $0-9
	MOVQ i+0(FP), AX
	LEAQ bytes<>+0(SB), CX
	MOVB (CX)(AX*1), AL
	MOVB AL, ret+8(FP)
	RET
