// Code generated by command: go run asm.go -out pragma.s -stubs stub.go. DO NOT EDIT.

#include "textflag.h"

// func Add(z *uint64, x *uint64, y *uint64)
TEXT ·Add(SB), NOSPLIT, $0-24
	MOVQ z+0(FP), AX
	MOVQ x+8(FP), CX
	MOVQ y+16(FP), DX
	MOVQ (CX), CX
	MOVQ (DX), DX
	ADDQ CX, DX
	MOVQ DX, (AX)
	RET
