// Code generated by command: go run asm.go -out args.s -stubs stub.go. DO NOT EDIT.

package args

func Second(x int32, y int32) int32

func StringLen(s string) int

func SliceLen(s []int) int

func SliceCap(s []int) int

func ArrayThree(a [7]uint64) uint64

func FieldByte(s Struct) byte

func FieldInt8(s Struct) int8

func FieldUint16(s Struct) uint16

func FieldInt32(s Struct) int32

func FieldUint64(s Struct) uint64

func FieldFloat32(s Struct) float32

func FieldFloat64(s Struct) float64

func FieldStringLen(s Struct) int

func FieldSliceCap(s Struct) int

func FieldArrayTwoBTwo(s Struct) byte

func FieldArrayOneC(s Struct) uint16

func FieldComplex64Imag(s Struct) float32

func FieldComplex128Real(s Struct) float64

func DereferenceFloat32(s *Struct) float32
