// Code generated by command: go run asm.go -out fmt.s -stubs stub.go. DO NOT EDIT.

#include "textflag.h"

// func Formatting()
TEXT ·Formatting(SB), NOSPLIT, $0
	ADDQ R8, R8

	// One comment line between instructions.
	ADDQ R8, R8

	// Comment before label.
label:
	// Comment after label.
	ADDQ R8, R8
	JMP  label
	RET
