// Code generated by command: go run asm.go -out issue76.s -stubs stub.go. DO NOT EDIT.

#include "textflag.h"

// func Issue76(x uint64, y uint64) uint64
TEXT ·Issue76(SB), NOSPLIT, $0-24
	MOVQ x+0(FP), AX
	MOVQ y+8(FP), CX
	ADDQ CX, AX
	MOVQ AX, ret+16(FP)
	RET
