// Code generated by annoyingaliases.sh. DO NOT EDIT.

package load

var annoyingaliases = []alias{
	{"JA", "JHI"},
	{"JAE", "JCC"},
	{"JB", "JCS"},
	{"JBE", "JLS"},
	{"JC", "JCS"},
	{"JE", "JEQ"},
	{"JG", "JGT"},
	{"JHS", "JCC"},
	{"JL", "JLT"},
	{"JLO", "JCS"},
	{"JNA", "JLS"},
	{"JNAE", "JCS"},
	{"JNB", "JCC"},
	{"JNBE", "JHI"},
	{"JNC", "JCC"},
	{"JNG", "JLE"},
	{"JNGE", "JLT"},
	{"JNL", "JGE"},
	{"JNLE", "JGT"},
	{"JNO", "JOC"},
	{"JNP", "JPC"},
	{"JNS", "JPL"},
	{"JNZ", "JNE"},
	{"JO", "JOS"},
	{"JP", "JPS"},
	{"JPE", "JPS"},
	{"JPO", "JPC"},
	{"JS", "JMI"},
	{"JZ", "JEQ"},
	{"MASKMOVDQU", "MASKMOVOU"},
	{"MOVD", "MOVQ"},
	{"MOVDQ2Q", "MOVQ"},
	{"MOVNTDQ", "MOVNTO"},
	{"MOVOA", "MOVO"},
	{"PSLLDQ", "PSLLO"},
	{"PSRLDQ", "PSRLO"},
	{"PADDD", "PADDL"},
}
