// Code generated by command: go run asm.go -out sha1.s -stubs stub.go. DO NOT EDIT.

package sha1

// block SHA-1 hashes the 64-byte message m into the running state h.
func block(h *[5]uint32, m []byte)
