package build

import "github.com/mmcloughlin/avo/attr"

// TEXT and DATA attribute values included for convenience.
const (
	NOPROF        = attr.NOPROF
	DUPOK         = attr.DUPOK
	NOSPLIT       = attr.NOSPLIT
	RODATA        = attr.RODATA
	NOPTR         = attr.NOPTR
	WRAPPER       = attr.WRAPPER
	NEEDCTXT      = attr.NEEDCTXT
	TLSBSS        = attr.TLSBSS
	NOFRAME       = attr.NOFRAME
	REFLECTMETHOD = attr.REFLECTMETHOD
	TOPFRAME      = attr.TOPFRAME
)
